export const maxLogSizeToParse = 1000000,
  maxLogSize = 2 * Math.pow(10, 6)

export const defaultLogThemes = {
  dark: {
    themeName: 'dark',
    debug: '#a6e22e',
    soql: '#66d9ef',
    callout: '#ae81ff',
    system: '#75715e',
    rest: '#f8f8f2',
    method: '#e6db74',
    error: '#f92672',
    background: '#292724'
  },
  light: {
    themeName: 'light',
    debug: '#0000FF',
    soql: '#AA22FF',
    callout: '#A0A000',
    system: '#008800',
    rest: '#880000',
    method: '#BB4444',
    error: '#f92672',
    background: '#ffffcc'
  },
  bw: {
    themeName: 'bw',
    debug: '#000',
    soql: '#000',
    callout: '#000',
    system: '#000',
    rest: '#000',
    method: '#000',
    error: '#000',
    background: '#fff'
  }
}

export const sfIdRegex = /\b[a-zA-Z0-9]{18}\b|\b[a-zA-Z0-9]{15}\b/g
export const logEventToType = {
  USER_DEBUG: 'debug',
  SYSTEM_: 'system',
  ENTERING_MANAGED_PKG: 'system',
  SOQL_EXECUTE_: 'soql',
  METHOD_: 'method',
  CONSTRUCTOR_: 'method',
  CODE_UNIT: 'method',
  EXCEPTION_: 'err',
  FATAL_ERROR: 'err',
  CALLOUT: 'callout',
  VALIDATION_: 'method',
  EXECUTION_: 'rest',
  DML_BEGIN: 'rest',
  DML_END: 'rest'
}

export const allEventTypes = [
  'BULK_HEAP_ALLOCATE',
  'CALLOUT_REQUEST',
  'CALLOUT_RESPONSE',
  'CODE_UNIT_FINISHED',
  'CODE_UNIT_STARTED',
  'CONSTRUCTOR_ENTRY',
  'CONSTRUCTOR_EXIT',
  'CUMULATIVE_LIMIT_USAGE',
  'CUMULATIVE_LIMIT_USAGE_END',
  'CUMULATIVE_PROFILING',
  'CUMULATIVE_PROFILING_BEGIN',
  'CUMULATIVE_PROFILING_END',
  'DML_BEGIN',
  'DML_END',
  'EMAIL_QUEUE',
  'ENTERING_MANAGED_PKG',
  'EVENT_SERVICE_PUB_BEGIN',
  'EVENT_SERVICE_PUB_DETAIL',
  'EVENT_SERVICE_PUB_END',
  'EVENT_SERVICE_SUB_BEGIN',
  'EVENT_SERVICE_SUB_DETAIL',
  'EVENT_SERVICE_SUB_END',
  'EXCEPTION_THROWN',
  'EXECUTION_FINISHED',
  'EXECUTION_STARTED',
  'FATAL_ERROR',
  'FLOW_ACTIONCALL_DETAIL',
  'FLOW_ASSIGNMENT_DETAIL',
  'FLOW_BULK_ELEMENT_BEGIN',
  'FLOW_BULK_ELEMENT_DETAIL',
  'FLOW_BULK_ELEMENT_END',
  'FLOW_CREATE_INTERVIEW_BEGIN',
  'FLOW_CREATE_INTERVIEW_END',
  'FLOW_CREATE_INTERVIEW_ERROR',
  'FLOW_ELEMENT_BEGIN',
  'FLOW_ELEMENT_DEFERRED',
  'FLOW_ELEMENT_END',
  'FLOW_ELEMENT_ERROR',
  'FLOW_ELEMENT_FAULT',
  'FLOW_INTERVIEW_PAUSED',
  'FLOW_INTERVIEW_RESUMED',
  'FLOW_LOOP_DETAIL',
  'FLOW_RULE_DETAIL',
  'FLOW_START_INTERVIEW_BEGIN',
  'FLOW_START_INTERVIEW_END',
  'FLOW_START_INTERVIEWS_BEGIN',
  'FLOW_START_INTERVIEWS_END',
  'FLOW_START_INTERVIEWS_ERROR',
  'FLOW_SUBFLOW_DETAIL',
  'FLOW_VALUE_ASSIGNMENT',
  'FLOW_WAIT_EVENT_RESUMING_DETAIL',
  'FLOW_WAIT_EVENT_WAITING_DETAIL',
  'FLOW_WAIT_RESUMING_DETAIL',
  'FLOW_WAIT_WAITING_DETAIL',
  'HEAP_ALLOCATE',
  'HEAP_DEALLOCATE',
  'IDEAS_QUERY_EXECUTE',
  'LIMIT_USAGE_FOR_NS',
  'METHOD_ENTRY',
  'METHOD_EXIT',
  'POP_TRACE_FLAGS',
  'PUSH_NOTIFICATION_INVALID_APP',
  'PUSH_NOTIFICATION_INVALID_CERTIFICATE',
  'PUSH_NOTIFICATION_INVALID_NOTIFICATION',
  'PUSH_NOTIFICATION_NO_DEVICES',
  'PUSH_NOTIFICATION_NOT_ENABLED',
  'PUSH_NOTIFICATION_SENT',
  'PUSH_TRACE_FLAGS',
  'QUERY_MORE_BEGIN',
  'QUERY_MORE_END',
  'QUERY_MORE_ITERATIONS',
  'SAVEPOINT_ROLLBACK',
  'SAVEPOINT_SET',
  'SLA_END',
  'SLA_EVAL_MILESTONE',
  'SLA_NULL_START_DATE',
  'SLA_PROCESS_CASE',
  'SOQL_EXECUTE_BEGIN',
  'SOQL_EXECUTE_END',
  'SOSL_EXECUTE_BEGIN',
  'SOSL_EXECUTE_END',
  'STACK_FRAME_VARIABLE_LIST',
  'STATEMENT_EXECUTE',
  'STATIC_VARIABLE_LIST',
  'SYSTEM_CONSTRUCTOR_ENTRY',
  'SYSTEM_CONSTRUCTOR_EXIT',
  'SYSTEM_METHOD_ENTRY',
  'SYSTEM_METHOD_EXIT',
  'SYSTEM_MODE_ENTER',
  'SYSTEM_MODE_EXIT',
  'TESTING_LIMITS',
  'TOTAL_EMAIL_RECIPIENTS_QUEUED',
  'USER_DEBUG',
  'VALIDATION_ERROR',
  'VALIDATION_FAIL',
  'VALIDATION_FORMULA',
  'VALIDATION_PASS',
  'VALIDATION_RULE',
  'VARIABLE_ASSIGNMENT',
  'VARIABLE_SCOPE_BEGIN',
  'VARIABLE_SCOPE_END',
  'VF_APEX_CALL',
  'VF_DESERIALIZE_VIEWSTATE_BEGIN',
  'VF_DESERIALIZE_VIEWSTATE_END',
  'VF_EVALUATE_FORMULA_BEGIN',
  'VF_EVALUATE_FORMULA_END',
  'VF_PAGE_MESSAGE',
  'VF_SERIALIZE_VIEWSTATE_BEGIN',
  'VF_SERIALIZE_VIEWSTATE_END',
  'WF_ACTION',
  'WF_ACTION_TASK',
  'WF_ACTIONS_END',
  'WF_APPROVAL',
  'WF_APPROVAL_REMOVE',
  'WF_APPROVAL_SUBMIT',
  'WF_ASSIGN',
  'WF_CRITERIA_BEGIN',
  'WF_CRITERIA_END',
  'WF_EMAIL_ALERT',
  'WF_EMAIL_SENT',
  'WF_ENQUEUE_ACTIONS',
  'WF_ESCALATION_ACTION',
  'WF_ESCALATION_RULE',
  'WF_EVAL_ENTRY_CRITERIA',
  'WF_FIELD_UPDATE',
  'WF_FLOW_ACTION_BEGIN',
  'WF_FLOW_ACTION_DETAIL',
  'WF_FLOW_ACTION_END',
  'WF_FLOW_ACTION_ERROR',
  'WF_FLOW_ACTION_ERROR_DETAIL',
  'WF_FORMULA',
  'WF_HARD_REJECT',
  'WF_NEXT_APPROVER',
  'WF_NO_PROCESS_FOUND',
  'WF_OUTBOUND_MSG',
  'WF_PROCESS_NODE',
  'WF_REASSIGN_RECORD',
  'WF_RESPONSE_NOTIFY',
  'WF_RULE_ENTRY_ORDER',
  'WF_RULE_EVAL_BEGIN',
  'WF_RULE_EVAL_END',
  'WF_RULE_EVAL_VALUE',
  'WF_RULE_FILTER',
  'WF_RULE_INVOCATION',
  'WF_RULE_NOT_EVALUATED',
  'WF_SOFT_REJECT',
  'WF_SPOOL_ACTION_BEGIN',
  'WF_TIME_TRIGGER',
  'WF_TIME_TRIGGERS_BEGIN',
  'XDS_RESPONSE',
  'XDS_RESPONSE_DETAIL',
  'XDS_RESPONSE_ERROR'
]
